/*
  ktracker (c) 2006 Kris Beevers

  This file is part of ktracker.

  ktracker is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  ktracker is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with ktracker; if not, write to the Free Software Foundation,
  Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

  $Id: ktrackerclient.cpp,v 1.1.1.1 2006/10/10 20:41:29 beevek Exp $
*/

#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <signal.h>
#include <inttypes.h>
#include <assert.h>
#include "ktrackerclient.hpp"

// most of this stuff was taken from librobot/net.c in my robot cvs
// repository

// actually i copied the net_* stuff and connect/disconnect directly
// from rbnet.cpp in ratbot-net-simple, which in turn originated in
// the robot cvs, which in turn was sort of familiar to old aimster
// code, not that any of it is particularly complicated.  you'd think
// i might get around to just putting it all in a library someday but
// then i'd have to ship the library around too, and after all, there
// are always little tweaks that need to be made for particular
// applications.  so whatever.  i hope you enjoyed my commentary.

// oh also, because i'll probably be linking this with librbnet i
// changed net_* to _net_* to prevent name conflicts.

int ktsock = -1;

// internal
struct sigaction _old_sig;
void _net_default_alarm(int sig) {}
int _net_start_timeout(uint32_t ms)
{
  struct itimerval itv;
  itv.it_value.tv_sec = ms / 1000;
  itv.it_value.tv_usec = (ms % 1000) * 1000;
  itv.it_interval.tv_sec = itv.it_interval.tv_usec = 0;
  struct sigaction alrm;
  memset(&alrm, 0, sizeof(alrm));
  alrm.sa_handler = _net_default_alarm;
  alrm.sa_flags = SA_RESETHAND;
  sigaction(SIGALRM, &alrm, &_old_sig);
  if(setitimer(ITIMER_REAL, &itv, 0) < 0) {
    sigaction(SIGALRM, &_old_sig, &alrm);
    return -1;
  }
  return 0;
}
void _net_stop_timeout()
{
  struct itimerval itv;
  memset(&itv, 0, sizeof(itv));
  setitimer(ITIMER_REAL, &itv, 0);
  struct sigaction tmp;
  sigaction(SIGALRM, &_old_sig, &tmp);
}

int ktracker_connect(const char *ip, uint16_t port)
{
  struct sockaddr_in sa;
  sa.sin_family = AF_INET;
  sa.sin_port = htons(port);
  struct hostent *he = (struct hostent *)gethostbyname(ip);
  if(!he) {
    uint32_t addr = inet_addr(ip);
    if(addr == INADDR_NONE)
      return -1;
    memcpy(&sa.sin_addr, &addr, sizeof(addr));
  } else
    sa.sin_addr = *((struct in_addr *)he->h_addr);

  ktsock = socket(AF_INET, SOCK_STREAM, 0);
  if(ktsock < 0)
    return -1;

  if(_net_start_timeout(2000) < 0)
    return -1;
  if(connect(ktsock, (struct sockaddr *)&sa, sizeof(struct sockaddr_in)) < 0) {
    _net_stop_timeout();
    close(ktsock);
    return -1;
  }
  _net_stop_timeout();
  return ktsock;
}

int ktracker_disconnect()
{
  if(ktsock >= 0)
    shutdown(ktsock, 2);
  ktsock = -1;
  return 0;
}

int ktracker_recv_packet(uint8_t *data, uint8_t sz)
{
  assert(data);
  if(ktsock < 0)
    return -1;

  if(_net_start_timeout(2000) < 0)
    return -1;
  if(recv(ktsock, data, sz, MSG_WAITALL) != sz)
    return -1;
  _net_stop_timeout();
  return 0;
}

const uint8_t cmd_get_pose = 0x00;
const uint8_t cmd_set_orig = 0x01;

int ktracker_get_pose(double *x, double *y, double *t)
{
  if(ktsock < 0)
    return -1;
  if(write(ktsock, &cmd_get_pose, 1) != 1)
    return -1;
  double pose[3];
  if(ktracker_recv_packet((uint8_t *)pose, 3*sizeof(double)) < 0)
    return -1;
  *x = pose[0];
  *y = pose[1];
  *t = pose[2];
  return 0;
}

int ktracker_set_origin_to_current_pose()
{
  return write(ktsock, &cmd_set_orig, 1);
}
